/*****************************************************
Chip type               : ATmega8
Program type            : Application
AVR Core Clock frequency: 8,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*****************************************************/

#include <mega8.h>
#include <stdint.h>
#include <stdio.h>
#include <delay.h>

#define DHT22       //       DHT22  DHT11

#define DHT_PORT        PORTD
#define DHT_DDR         DDRD
#define DHT_PIN         PIND
#define DHT_BIT         1

#asm
   .equ __lcd_port=0x12 //PORTD
#endasm
// Alphanumeric LCD Module functions
#include <alcd.h>

unsigned char buff [16];  // 
 float h=0;
 float t=0;
 bit hum=0;
 uint8_t data[5]={0,0,0,0,0};

int read_dht_hum()   //   
{

unsigned char i,j;

//=============MCU send START
DHT_DDR|=(1<<DHT_BIT); //pin as output
DHT_PORT&=~(1<<DHT_BIT); //0
delay_ms(18);
DHT_PORT|=(1<<DHT_BIT); //1
DHT_DDR&=~(1<<DHT_BIT); //pin as input
//=============check DHT11 response
delay_us(50);
if (DHT_PIN&(1<<DHT_BIT))
    {
    return 0;
    }
delay_us(80);
if (!(DHT_PIN&(1<<DHT_BIT)))
    {
    return 0;
    }
//===============receive 40 data bits
while (DHT_PIN&(1<<DHT_BIT));
for (j=0; j<5; j++)
    {
    data[j]=0;
    for(i=0; i<8; i++)
        {
        while (!(DHT_PIN&(1<<DHT_BIT)));
        delay_us (30);
        if (DHT_PIN&(1<<DHT_BIT))
            data[j]|=1<<(7-i);
        while (DHT_PIN&(1<<DHT_BIT));
        }
    }

return 1; 
//DHT11 data[0]-humidity, data[2]-temperature 
//DHT22 data[0]-data[1]-humidity, data[2]-data[3]-temperature
}

void dht_read()
{
   hum=0;
       #asm("cli")
        hum = read_dht_hum();
       #asm("sei")
    if(hum==1)
    { 
      #ifdef DHT22
      h = (float) (data[0]<<8);
      h = h + data[1];
      h = h/10;
      t = (float) (data[2]<<8);
      t = t + data[3];
      if(t<0) //   0
     {
      t = t+32768; // 
      t = -t; //  
     }
     t = t/10;  
     #else
     h =  data[0];
     t =  data[2];
     #endif
     }
}


void pr()
{
   dht_read();
    lcd_gotoxy(0,0);              //
   sprintf(buff,"MEGA-AVR.COM.UA ");
   lcd_puts(buff);
   lcd_gotoxy(0,1);              //  
   #ifdef DHT22
   sprintf(buff,"DHT22 %.1f%C/%.1f%%  ",t,h);
   #else
   sprintf(buff,"DHT11 %.1f%C/%.1f%%  ",t,h);
   #endif
   lcd_puts(buff);
}

void main(void)
{
PORTB=0x00;
DDRB=0xF0;

PORTC=0x00;
DDRC=0x00;

PORTD=0x00;
DDRD=0xE0;

lcd_init(16);

while (1)
      {
       pr();
      }
}
